function [A,B,C,D,E]=lpvsys_harald(p)
% Function to compute the LPV state-space matrices for Harald's system 
% (LPV plant+gain scheduling PI controller)

if isempty(p),
    Range = [2 7];
    Rate  = [-1 1];
    A=2; B=1; C=1; D=Range; E=Rate; %dimension of state,input and output
    return;
end;

% Parameter-dependent gain and time-constant for plant
tau=sqrt(133.6-16.8*p);
K=sqrt(4.8*p-8.6);

% PI gains for controller
Kp=(2*0.7*0.25*tau-1)/K;
Ki=0.25^2*tau/K;

% State-space matrices for closed-loop sensitivity function
A=[-1/tau*(1+Kp*K) 1/tau; -Ki*K 0];
B=[1/tau*Kp;Ki];
C=[-K 0]; 
D=1;


