function [nu_opt,x_opt,x_init]=max_nu(lpvsys,ptrajectory,tdata,g,solver,c0,hRange,MaxIter,opts)
% Maximizing nu, where nu is the induced L2 norm of the discrete-time LTI
% system associated with the periodic LTV system obtained by evaluating the 
% LPV system over a periodic scheduling trajectory.

[~,~,~,pRange,pRate]=lpvsys([]);

info=functions(ptrajectory);
switch info.function
    case 'traj_pwcder'
        [Alp,Blp,Aeq]=ptrajectory([],[],tdata,1,pRange,hRange);
        L=null(Aeq);
        Alp=Alp*L; 
        P=Polyhedron(Alp,Blp); dimP=size(P.H,2)-1;
        if dimP<20, P.minHRep(); end;
        if dimP<4, plot(P,'Wire',true,'LineWidth',2); hold on; drawnow; end;
        if isempty(c0), 
            c0=cprnd(1,P.H(:,1:end-1),P.H(:,end)); c0=c0';             
        else
            c0=L\c0;            
        end
        B=P.outerApprox;
        x_U=B.H(1:dimP,end);
        x_L=-B.H(dimP+1:end,end);
        b_L=-inf*ones(size(P.H,1),1);
        b_U=P.H(:,end);
        A=P.H(:,1:end-1);
        
     
        switch solver
            case 'patternsearch'
                if isempty(opts), opts=psoptimset(@patternsearch); end;
                opts = psoptimset(opts,'Display','iter',...
                        'MaxIter',MaxIter,'TolFun',1e-5,'TolMesh',1e-5,'TolX',1e-5);
       
                
                tic;
                [c_opt, nu_opt] = patternsearch(@(c) -obj_nu(L*c,g,lpvsys,ptrajectory,tdata),...
                                  c0,A,b_U,[],[],x_L,x_U,[],opts);
                toc;
                nu_opt=-nu_opt;
                c_opt=L*c_opt;
                x_opt=c_opt;
                x_init=L*c0; %just giving back the initial value
            otherwise, error('Unknown solver.'); 
        end;
    case 'traj_sinusoids'
        [Alp,Blp]=ptrajectory([],[],tdata,1,pRange,pRate,hRange);
        P=Polyhedron(Alp,Blp); dimP=size(P.H,2)-1;
        if dimP<20, P.minHRep(); end;
        if dimP<4, plot(P,'Wire',true,'LineWidth',2); hold on; drawnow; end;
        if isempty(c0), c0=cprnd(1,P.H(:,1:end-1),P.H(:,end)); c0=c0'; end
        B=P.outerApprox;
        x_U=B.H(1:dimP,end);
        x_L=-B.H(dimP+1:end,end);
        b_L=-inf*ones(size(P.H,1),1);
        b_U=P.H(:,end);
        A=P.H(:,1:end-1);
        
        switch solver
            case 'patternsearch'
                if isempty(opts), opts=psoptimset(@patternsearch); end;
                opts = psoptimset(opts,'Display','iter',...
                        'MaxIter',MaxIter,'TolFun',1e-5,'TolMesh',1e-5,'TolX',1e-5);
                tic;    
                [c_opt, nu_opt] = patternsearch(@(c) -obj_nu(c,g,lpvsys,ptrajectory,tdata),...
                                  c0,A,b_U,[],[],x_L,x_U,[],opts);
                toc;
                nu_opt=-nu_opt;
                x_opt=c_opt;
                x_init=c0;
            otherwise, error('Unknown solver.'); 
        end
     
        
        
    otherwise, error('Unknown trajectory structure.');
end;
        



