clear all; close all;

%switching off annoying warnings of ode45
warning('off','MATLAB:ode45:IntegrationTolNotMet');


%% Select an example 

example='1a';

switch example,
    case '1a'
        lpvsys=@lpvsys_haraldcatws;
        ptrajectory=@traj_pwcder2p;  
        fmp=1; msk={[1 0 -1 0 1 0] [0 1 0 -1 0 1 0]}; gU=5.4; hRange=[0.1 20;0.1 20]; cfixed=[];
        filename='harald_catws_'; fileidx=0; solver='patternsearch_3';  MaxIter=50; 
    
end; 

directory='./matfiles/';


while 1,
    
%% Loading the initial value
if fileidx>0,
    file=load([directory filename sprintf('%02d',fileidx)],'c');
    c0=file.c;
    
    i=1;
    while i>0,
        str=[directory filename sprintf('%02d',fileidx+i) '.mat']; 
        if exist(str,'file'), 
            delete(str); i=i+1;
        else
            i=0;
        end;
    end;
         
else
    delete([directory filename '*.mat']);
    if ~exist('c0','var'), c0=[]; end;
end;
fileidx=fileidx+1;    


n1=length(msk{1})+1; n2=length(msk{2})+1;
switch str2double(solver(end)),
    case 2, cfixed=c0(1:n1); c0=c0(n1+1:end);
    case 1, cfixed=c0(n1+1:end); c0=c0(1:n1);
end;


%% Optimization
[nu_opt,c_opt]=max_nu(lpvsys,ptrajectory,fmp,msk,gU,solver,c0,hRange,cfixed,MaxIter)
c=c_opt; 
h=ptrajectory([],c,fmp,msk,-1); w=2*pi/h;


%% Computing gamma
verbose=1;
[gL,gL_high]=periodicL2gain(@(t) persys(t,lpvsys,ptrajectory,c,fmp,msk),verbose)

%% Plotting the trajectory
figure; 
Nt = 2000; h=2*pi/w;
t = linspace(0,3*h,Nt);
[p,dp]=ptrajectory(t,c,fmp,msk);
subplot(2,1,1); plot(t,p,'LineWidth',2); ylabel('\rho(t)');
subplot(2,1,2); plot(t,dp,'LineWidth',2); ylabel('d\rho(t)/dt');
hold on;
grid on; axis tight;
xlabel('t');

%% Saving the result
save([directory filename sprintf('%02d',fileidx)]);
reply=input('Continue?','s');
if reply~='y' && reply~='Y', break; end;

end;







