function [time,yval,xval]=simlpvsys(lpvsys,ptrajectory,c,fmp,msk,x0,tspan,utime,uval)
%Simulates the LPV system lpvsys over tspan with scheduling trajectory defined by
%(c,ptrajectory,fmp,msk) and input given in (uval,utime). 

[~,~,ny]=lpvsys([]);

% Set options
options = odeset('RelTol',1e-7,'AbsTol',1e-10);

xdotfh = @(t,x) xdotfun(t,x,lpvsys,ptrajectory,c,fmp,msk,utime,uval);
[time,xval]=ode45(xdotfh,tspan,x0,options);

N=length(time);
yval=zeros(N,ny);
for i=1:N
    p = ptrajectory(time(i),c,fmp,msk);
    [~,~,C,D]=lpvsys(p);
    u=interp1(utime,uval,time(i),'linear');
    yval(i,:)=C*xval(i,:)'+D*u(:);
end;




function xdot=xdotfun(t,x,lpvsys,ptrajectory,c,fmp,msk,utime,uval)
% Construct parameter vector at time t from basis functions
p = ptrajectory(t,c,fmp,msk);

% Evaluate state-space matrices of LPV system at current parameter vector
[A,B,~,~]=lpvsys(p);

% xdot
u=interp1(utime,uval,t);
xdot=A*x+B*u(:);
