function [out1,out2,out3,out4]=traj_pwcder2p(t,c,fmp,msk,task,Range,hRange,cfixed)

if nargin==0
    close all
    
    fmp=1
    msk={0.2*[1 0 -1 0] 0.2*[0 -1 0 1]}; 
    Range=[-0.2 0.2;-0.2 0.2]
    hRange=[1 50;1 50]; cfixed=[];
    c=[10 5 10 5 -1 5 10 5 10 1]';
    
    %[Alp,Blp,Aeq]=pwder([],[],[],[],msk,1,Range,hRange)
      
    %c=[1 2 3 4 2 2 10, 1 2 1 1 0]';
    %c1=[1 2 3 4 2 2 10]';
    %c2=[1 2 1 1 0]';
    t=0:0.001:40; 
    [p,pdot]=traj_pwcder2p(t,c,fmp,msk);
    subplot(2,1,1); plot(t,p)
    subplot(2,1,2); plot(t,pdot)
    
    traj_pwcder2p([],c,fmp,msk,-1)
    
    %[Alp,Blp,Aeq,beq]=traj_pwcder2p([],[],fmp,msk,2,Range,hRange,c1)
    %Aeq\beq
  
    
    return
end

if nargin==4, task=0; end;


switch task,
    case -1, %computing the period 
        n=[length(msk{1}) length(msk{2})];
        h1=sum(c(1:n(1)));
        h2=sum(c(n(1)+2:n(1)+1+n(2)));
        if h1==0, h=h2; 
        elseif h2==0, h=h1; 
        elseif fmp==0, h=h1*h2; 
        else h=max(h1,h2); 
        end;
        out1=h; out2=[]; out3=[]; out4=[];
        
        
    case 0, %evaluating the parameter at time t
        l=0; c=c(:)'; Nt=numel(t); p=zeros(2,Nt); pdot=p;
        for i=1:2
            derivs=msk{i};
            n=length(derivs);
            tau=c(l+1:l+n); p0=c(l+n+1);
            taucs=cumsum(tau);
            const=cumsum([p0 tau.*derivs]);
            h=taucs(end); 
            if h==0, p(i,:)=p0; continue; end;
            t1=mod(t,h);
            for j=1:Nt
                k=find(t1(j)<=taucs,1);
                if k>1, t2=t1(j)-taucs(k-1); else t2=t1(j); end;
                p(i,j)=const(k)+derivs(k)*t2;
                pdot(i,j)=derivs(k);  
            end;
            l=l+n+1;
        end;
        out1=p; out2=pdot; out3=[]; out4=[];
           
    case 1, %lin. constraints if only the first parameter is tuned
        
        %(a) Range constraints for the parameter trajectory
        A1=[]; b1=[];
        derivs=msk{1}; n=length(derivs);
        M=zeros(n,n+1);
        k=0;
        for j=1:n
            if derivs(j)~=0 
                k=k+1; M(k,1:j)=derivs(1:j); M(k,end)=1; 
            end;
        end;
        M(k+1:end,:)=[]; 
        A1=[M;-M]; 
        b1=[b1;Range(1,2)*ones(k,1);-Range(1,1)*ones(k,1)];
        
        %(b) constraint for the period
        if fmp>0
            Aeq1=[1/fmp*ones(1,n) 0]; beq1=sum(cfixed(1:end-1)); A2=[]; b2=[];
            %--the period should be equal to the 1/fmp*h2, where h2 is the
            %period of the fixed second trajectory
        else
            E=[ones(1,n)  0];
            A2=[E;-E]; b2=[hRange(1,2);-hRange(1,1)];
            Aeq1=[]; beq1=[];
            %--the period can be tuned freely between the given bounds
        end;
        
        %(c) the length of the intervals has to be positive
        A3=-[eye(n) zeros(n,1)]; b3=zeros(n,1);
        
        %(d) the values of the parameter at the beginning and at the end of
        %the period should be equal
        Aeq2=[derivs 0]; beq2=0;
        
        A=[A1;A2;A3]; b=[b1;b2;b3]; Aeq=[Aeq1;Aeq2]; beq=[beq1;beq2];
        out1=A; out2=b; out3=Aeq; out4=beq;
        
    case 2, %lin. constraints if only the second parameter is tuned
        
        %(a) Range constraints for the parameter trajectory
        A1=[]; b1=[];
        derivs=msk{2}; n=length(derivs);
        M=zeros(n,n+1);
        k=0;
        for j=1:n
            if derivs(j)~=0 
                k=k+1; M(k,1:j)=derivs(1:j); M(k,end)=1; 
            end;
        end;
        M(k+1:end,:)=[]; 
        A1=[M;-M]; 
        b1=[b1;Range(2,2)*ones(k,1);-Range(2,1)*ones(k,1)];
        
        %(b) constraint for the period
        if fmp>0
            Aeq1=[fmp*ones(1,n) 0]; beq1=sum(cfixed(1:end-1)); A2=[]; b2=[];
            %--the period should be equal to the fmp*h1, where h1 is the
            %period of the fixed first trajectory
        else
            E=[ones(1,n)  0];
            A2=[E;-E]; b2=[hRange(2,2);-hRange(2,1)];
            Aeq1=[]; beq1=[];
            %--the period can be tuned freely between the given bounds
        end;
        
        %(c) the length of the intervals has to be positive
        A3=-[eye(n) zeros(n,1)]; b3=zeros(n,1);
        
        
        %(d) the values of the parameter at the beginning and at the end of
        %the period should be equal
        Aeq2=[derivs 0]; beq2=0;
        
        A=[A1;A2;A3]; b=[b1;b2;b3]; Aeq=[Aeq1;Aeq2]; beq=[beq1;beq2];
        out1=A; out2=b; out3=Aeq; out4=beq;
        
            
            

        
    case 3,
        % Form linear constraints to specify the allowable parameter trajectory
        
        %(a) Range constraints for the parameter trajectories
        A1=[]; b1=[];
        for i=1:2
            derivs=msk{i};
            n=length(derivs);
            M=zeros(n,n+1);
            k=0;
            for j=1:n
                if derivs(j)~=0 
                    k=k+1; M(k,1:j)=derivs(1:j); M(k,end)=1; 
                end;
            end;
            M(k+1:end,:)=[]; 
            A1=blkdiag(A1,[M;-M]); 
            b1=[b1;Range(i,2)*ones(k,1);-Range(i,1)*ones(k,1)];
        end;
        
        %(b) constraint for the period
        n=[length(msk{1}) length(msk{2})];
        E=[ones(1,n(1))  0 zeros(1,n(2)) 0];
        A2=[E;-E]; b2=[hRange(1,2);-hRange(1,1)];
            %--the period of the first traj. is tuned freely between the
            %given bounds
        
        if fmp>0
            Aeq1=[-fmp*ones(1,n(1)) 0 ones(1,n(2)) 0]; A3=[]; b3=[];
                %--the period of the second traj. should be equal to
                %fmp*h1, where h1 is the period of the first traj.
        else
            E=[zeros(1,n(1))  0 ones(1,n(2)) 0];
            A3=[E;-E]; b3=[hRange(2,2);-hRange(2,1)];
            Aeq1=[];
                %--the period of the second traj. is freely tuned between
                %the given bounds
        end;
            
        %(c) the length of each interval sould be positive    
        A3=-blkdiag([eye(n(1)) zeros(n(1),1)],[eye(n(2)) zeros(n(2),1)]);
        b3=zeros(sum(n),1);
        
        %(d) the values of the parameters at the beginning and at the end of
        %the period should be equal
        Aeq2=blkdiag([msk{1} 0],[msk{2} 0]);
        
        A=[A1;A2;A3]; b=[b1;b2;b3]; Aeq=[Aeq1;Aeq2]; beq=zeros(size(Aeq,1),1);
        out1=A; out2=b; out3=Aeq; out4=beq;
        
        
end;
        
        

